import { Registry } from 'vscode/vscode/vs/platform/registry/common/platform';
import { localizeWithPath, localize2WithPath } from 'vscode/vscode/vs/nls';
import { EditorPaneDescriptor } from 'vscode/vscode/vs/workbench/browser/editor';
import { EditorExtensions } from 'vscode/vscode/vs/workbench/common/editor';
import { IsAuxiliaryEditorPartContext, ActiveEditorGroupLockedContext, EditorPartMultipleEditorGroupsContext, MultipleEditorGroupsContext, InEditorZenModeContext, EditorGroupEditorsCountContext, ActiveEditorLastInGroupContext, EditorTabsVisibleContext, ActiveEditorAvailableEditorIdsContext, ActiveEditorPinnedContext, ActiveEditorStickyContext, ActiveEditorCanSplitInGroupContext, SideBySideEditorActiveContext, EditorPartMaximizedEditorGroupContext, ActiveEditorDirtyContext, TextCompareEditorActiveContext, ActiveCompareEditorCanSwapContext } from 'vscode/vscode/vs/workbench/common/contextkeys';
import { SideBySideEditorInput, SideBySideEditorInputSerializer } from 'vscode/vscode/vs/workbench/common/editor/sideBySideEditorInput';
import { TextResourceEditor } from 'vscode/vscode/vs/workbench/browser/parts/editor/textResourceEditor';
import { SideBySideEditor } from 'vscode/vscode/vs/workbench/browser/parts/editor/sideBySideEditor';
import { DiffEditorInput, DiffEditorInputSerializer } from 'vscode/vscode/vs/workbench/common/editor/diffEditorInput';
import { UntitledTextEditorInput } from 'vscode/vscode/vs/workbench/services/untitled/common/untitledTextEditorInput';
import { TextResourceEditorInput } from 'vscode/vscode/vs/workbench/common/editor/textResourceEditorInput';
import { TextDiffEditor } from 'vscode/vscode/vs/workbench/browser/parts/editor/textDiffEditor';
import { BinaryResourceDiffEditor } from 'vscode/vscode/vs/workbench/browser/parts/editor/binaryDiffEditor';
import { EditorStatusContribution, ChangeLanguageAction, ChangeEOLAction, ChangeEncodingAction } from 'vscode/vscode/vs/workbench/browser/parts/editor/editorStatus';
import { Categories } from 'vscode/vscode/vs/platform/action/common/actionCommonCategories';
import { registerAction2, MenuRegistry, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { SyncDescriptor } from 'vscode/vscode/vs/platform/instantiation/common/descriptors';
import { ShowEditorsInActiveGroupByMostRecentlyUsedAction, ShowAllEditorsByAppearanceAction, ShowAllEditorsByMostRecentlyUsedAction, NavigateForwardAction, NavigateBackwardsAction, OpenNextEditor, OpenPreviousEditor, OpenNextEditorInGroup, OpenPreviousEditorInGroup, OpenFirstEditorInGroup, OpenLastEditorInGroup, OpenNextRecentlyUsedEditorAction, OpenPreviousRecentlyUsedEditorAction, OpenNextRecentlyUsedEditorInGroupAction, OpenPreviousRecentlyUsedEditorInGroupAction, ReopenClosedEditorAction, ClearRecentFilesAction, CloseAllEditorsAction, CloseAllEditorGroupsAction, CloseLeftEditorsInGroupAction, CloseEditorsInOtherGroupsAction, CloseEditorInAllGroupsAction, RevertAndCloseEditorAction, SplitEditorAction, SplitEditorOrthogonalAction, SplitEditorLeftAction, SplitEditorRightAction, SplitEditorUpAction, SplitEditorDownAction, JoinTwoGroupsAction, JoinAllGroupsAction, NavigateBetweenGroupsAction, ResetGroupSizesAction, ToggleGroupSizesAction, MaximizeGroupHideSidebarAction, ToggleMaximizeEditorGroupAction, MinimizeOtherGroupsAction, MinimizeOtherGroupsHideSidebarAction, MoveEditorLeftInGroupAction, MoveEditorRightInGroupAction, MoveGroupLeftAction, MoveGroupRightAction, MoveGroupUpAction, MoveGroupDownAction, DuplicateGroupLeftAction, DuplicateGroupRightAction, DuplicateGroupUpAction, DuplicateGroupDownAction, MoveEditorToPreviousGroupAction, MoveEditorToNextGroupAction, MoveEditorToFirstGroupAction, MoveEditorToLastGroupAction, MoveEditorToLeftGroupAction, MoveEditorToRightGroupAction, MoveEditorToAboveGroupAction, MoveEditorToBelowGroupAction, SplitEditorToPreviousGroupAction, SplitEditorToNextGroupAction, SplitEditorToFirstGroupAction, SplitEditorToLastGroupAction, SplitEditorToLeftGroupAction, SplitEditorToRightGroupAction, SplitEditorToAboveGroupAction, SplitEditorToBelowGroupAction, FocusActiveGroupAction, FocusFirstGroupAction, FocusLastGroupAction, FocusPreviousGroup, FocusNextGroup, FocusLeftGroup, FocusRightGroup, FocusAboveGroup, FocusBelowGroup, NewEditorGroupLeftAction, NewEditorGroupRightAction, NewEditorGroupAboveAction, NewEditorGroupBelowAction, NavigatePreviousAction, NavigateForwardInEditsAction, NavigateBackwardsInEditsAction, NavigatePreviousInEditsAction, NavigateToLastEditLocationAction, NavigateForwardInNavigationsAction, NavigateBackwardsInNavigationsAction, NavigatePreviousInNavigationsAction, NavigateToLastNavigationLocationAction, ClearEditorHistoryAction, EditorLayoutSingleAction, EditorLayoutTwoColumnsAction, EditorLayoutThreeColumnsAction, EditorLayoutTwoRowsAction, EditorLayoutThreeRowsAction, EditorLayoutTwoByTwoGridAction, EditorLayoutTwoRowsRightAction, EditorLayoutTwoColumnsBottomAction, ToggleEditorTypeAction, ReOpenInTextEditorAction, QuickAccessPreviousRecentlyUsedEditorAction, QuickAccessLeastRecentlyUsedEditorAction, QuickAccessPreviousRecentlyUsedEditorInGroupAction, QuickAccessLeastRecentlyUsedEditorInGroupAction, QuickAccessPreviousEditorFromHistoryAction, MoveEditorToNewWindowAction, CopyEditorToNewindowAction, MoveEditorGroupToNewWindowAction, CopyEditorGroupToNewWindowAction, RestoreEditorsToMainWindowAction, NewEmptyEditorWindowAction } from 'vscode/vscode/vs/workbench/browser/parts/editor/editorActions';
import { setup, LOCK_GROUP_COMMAND_ID, UNLOCK_GROUP_COMMAND_ID, CLOSE_EDITOR_GROUP_COMMAND_ID, SPLIT_EDITOR_UP, SPLIT_EDITOR_DOWN, SPLIT_EDITOR_LEFT, SPLIT_EDITOR_RIGHT, NEW_EMPTY_EDITOR_WINDOW_COMMAND_ID, TOGGLE_LOCK_GROUP_COMMAND_ID, MOVE_EDITOR_GROUP_INTO_NEW_WINDOW_COMMAND_ID, COPY_EDITOR_GROUP_INTO_NEW_WINDOW_COMMAND_ID, CLOSE_EDITOR_COMMAND_ID, CLOSE_OTHER_EDITORS_IN_GROUP_COMMAND_ID, CLOSE_EDITORS_TO_THE_RIGHT_COMMAND_ID, CLOSE_SAVED_EDITORS_COMMAND_ID, CLOSE_EDITORS_IN_GROUP_COMMAND_ID, REOPEN_WITH_COMMAND_ID, KEEP_EDITOR_COMMAND_ID, PIN_EDITOR_COMMAND_ID, UNPIN_EDITOR_COMMAND_ID, SPLIT_EDITOR_IN_GROUP, JOIN_EDITOR_IN_GROUP, MOVE_EDITOR_INTO_NEW_WINDOW_COMMAND_ID, COPY_EDITOR_INTO_NEW_WINDOW_COMMAND_ID, SHOW_EDITORS_IN_GROUP, TOGGLE_KEEP_EDITORS_COMMAND_ID, TOGGLE_MAXIMIZE_EDITOR_GROUP, SPLIT_EDITOR, TOGGLE_SPLIT_EDITOR_IN_GROUP_LAYOUT, CLOSE_PINNED_EDITOR_COMMAND_ID, CLOSE_EDITORS_AND_GROUP_COMMAND_ID, FOCUS_FIRST_SIDE_EDITOR, FOCUS_SECOND_SIDE_EDITOR } from 'vscode/vscode/vs/workbench/browser/parts/editor/editorCommands';
import { TOGGLE_DIFF_SIDE_BY_SIDE, GOTO_PREVIOUS_CHANGE, GOTO_NEXT_CHANGE, DIFF_SWAP_SIDES, TOGGLE_DIFF_IGNORE_TRIM_WHITESPACE } from 'vscode/vscode/vs/workbench/browser/parts/editor/diffEditorCommands';
import { inQuickPickContext, getQuickNavigateHandler } from 'vscode/vscode/vs/workbench/browser/quickaccess';
import { KeybindingsRegistry } from 'vscode/vscode/vs/platform/keybinding/common/keybindingsRegistry';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { isMacintosh } from 'vscode/vscode/vs/base/common/platform';
import { registerEditorContribution } from 'vscode/vscode/vs/editor/browser/editorExtensions';
import { FloatingEditorClickMenu } from 'vscode/vscode/vs/workbench/browser/codeeditor';
import { registerWorkbenchContribution2 } from 'vscode/vscode/vs/workbench/common/contributions';
import { Extensions } from 'vscode/vscode/vs/platform/quickinput/common/quickAccess';
import { ActiveGroupEditorsByMostRecentlyUsedQuickAccess, AllEditorsByAppearanceQuickAccess, AllEditorsByMostRecentlyUsedQuickAccess } from 'vscode/vscode/vs/workbench/browser/parts/editor/editorQuickAccess';
import { FileAccess } from 'vscode/vscode/vs/base/common/network';
import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { registerIcon } from 'vscode/vscode/vs/platform/theme/common/iconRegistry';
import { UntitledTextEditorInputSerializer, UntitledTextEditorWorkingCopyEditorHandler } from '../../../services/untitled/common/untitledTextEditorHandler.js';
import { DynamicEditorConfigurations } from './editorConfiguration.js';
import { ShowMultipleEditorTabsAction, ShowSingleEditorTabAction, HideEditorTabsAction, ZenShowMultipleEditorTabsAction, ZenShowSingleEditorTabAction, ZenHideEditorTabsAction, EditorActionsDefaultAction, EditorActionsTitleBarAction, HideEditorActionsAction } from 'vscode/vscode/vs/workbench/browser/actions/layoutActions';
import { EditorContextKeys } from 'vscode/vscode/vs/editor/common/editorContextKeys';
import 'vscode/vscode/vs/workbench/browser/parts/editor/editor.contribution._autosave';

const _moduleId = "vs/workbench/browser/parts/editor/editor.contribution";
( (Registry.as(EditorExtensions.EditorPane))).registerEditorPane(EditorPaneDescriptor.create(TextResourceEditor, TextResourceEditor.ID, ( localizeWithPath(_moduleId, 0, "Text Editor"))), [
    ( (new SyncDescriptor(UntitledTextEditorInput))),
    ( (new SyncDescriptor(TextResourceEditorInput)))
]);
( (Registry.as(EditorExtensions.EditorPane))).registerEditorPane(EditorPaneDescriptor.create(TextDiffEditor, TextDiffEditor.ID, ( localizeWithPath(_moduleId, 1, "Text Diff Editor"))), [
    ( (new SyncDescriptor(DiffEditorInput)))
]);
( (Registry.as(EditorExtensions.EditorPane))).registerEditorPane(EditorPaneDescriptor.create(BinaryResourceDiffEditor, BinaryResourceDiffEditor.ID, ( localizeWithPath(_moduleId, 2, "Binary Diff Editor"))), [
    ( (new SyncDescriptor(DiffEditorInput)))
]);
( (Registry.as(EditorExtensions.EditorPane))).registerEditorPane(EditorPaneDescriptor.create(SideBySideEditor, SideBySideEditor.ID, ( localizeWithPath(_moduleId, 3, "Side by Side Editor"))), [
    ( (new SyncDescriptor(SideBySideEditorInput)))
]);
( (Registry.as(EditorExtensions.EditorFactory))).registerEditorSerializer(UntitledTextEditorInput.ID, UntitledTextEditorInputSerializer);
( (Registry.as(EditorExtensions.EditorFactory))).registerEditorSerializer(SideBySideEditorInput.ID, SideBySideEditorInputSerializer);
( (Registry.as(EditorExtensions.EditorFactory))).registerEditorSerializer(DiffEditorInput.ID, DiffEditorInputSerializer);
registerWorkbenchContribution2(EditorStatusContribution.ID, EditorStatusContribution, 2 );
registerWorkbenchContribution2(UntitledTextEditorWorkingCopyEditorHandler.ID, UntitledTextEditorWorkingCopyEditorHandler, 2 );
registerWorkbenchContribution2(DynamicEditorConfigurations.ID, DynamicEditorConfigurations, 2 );
registerEditorContribution(FloatingEditorClickMenu.ID, FloatingEditorClickMenu, 1 );
const quickAccessRegistry = ( (Registry.as(Extensions.Quickaccess)));
const editorPickerContextKey = 'inEditorsPicker';
const editorPickerContext = ( (ContextKeyExpr.and(
    inQuickPickContext,
     (ContextKeyExpr.has(editorPickerContextKey))
)));
quickAccessRegistry.registerQuickAccessProvider({
    ctor: ActiveGroupEditorsByMostRecentlyUsedQuickAccess,
    prefix: ActiveGroupEditorsByMostRecentlyUsedQuickAccess.PREFIX,
    contextKey: editorPickerContextKey,
    placeholder: ( localizeWithPath(_moduleId, 4, "Type the name of an editor to open it.")),
    helpEntries: [{ description: ( localizeWithPath(_moduleId, 5, "Show Editors in Active Group by Most Recently Used")), commandId: ShowEditorsInActiveGroupByMostRecentlyUsedAction.ID }]
});
quickAccessRegistry.registerQuickAccessProvider({
    ctor: AllEditorsByAppearanceQuickAccess,
    prefix: AllEditorsByAppearanceQuickAccess.PREFIX,
    contextKey: editorPickerContextKey,
    placeholder: ( localizeWithPath(_moduleId, 4, "Type the name of an editor to open it.")),
    helpEntries: [{ description: ( localizeWithPath(_moduleId, 6, "Show All Opened Editors By Appearance")), commandId: ShowAllEditorsByAppearanceAction.ID }]
});
quickAccessRegistry.registerQuickAccessProvider({
    ctor: AllEditorsByMostRecentlyUsedQuickAccess,
    prefix: AllEditorsByMostRecentlyUsedQuickAccess.PREFIX,
    contextKey: editorPickerContextKey,
    placeholder: ( localizeWithPath(_moduleId, 4, "Type the name of an editor to open it.")),
    helpEntries: [{ description: ( localizeWithPath(_moduleId, 7, "Show All Opened Editors By Most Recently Used")), commandId: ShowAllEditorsByMostRecentlyUsedAction.ID }]
});
registerAction2(ChangeLanguageAction);
registerAction2(ChangeEOLAction);
registerAction2(ChangeEncodingAction);
registerAction2(NavigateForwardAction);
registerAction2(NavigateBackwardsAction);
registerAction2(OpenNextEditor);
registerAction2(OpenPreviousEditor);
registerAction2(OpenNextEditorInGroup);
registerAction2(OpenPreviousEditorInGroup);
registerAction2(OpenFirstEditorInGroup);
registerAction2(OpenLastEditorInGroup);
registerAction2(OpenNextRecentlyUsedEditorAction);
registerAction2(OpenPreviousRecentlyUsedEditorAction);
registerAction2(OpenNextRecentlyUsedEditorInGroupAction);
registerAction2(OpenPreviousRecentlyUsedEditorInGroupAction);
registerAction2(ReopenClosedEditorAction);
registerAction2(ClearRecentFilesAction);
registerAction2(ShowAllEditorsByAppearanceAction);
registerAction2(ShowAllEditorsByMostRecentlyUsedAction);
registerAction2(ShowEditorsInActiveGroupByMostRecentlyUsedAction);
registerAction2(CloseAllEditorsAction);
registerAction2(CloseAllEditorGroupsAction);
registerAction2(CloseLeftEditorsInGroupAction);
registerAction2(CloseEditorsInOtherGroupsAction);
registerAction2(CloseEditorInAllGroupsAction);
registerAction2(RevertAndCloseEditorAction);
registerAction2(SplitEditorAction);
registerAction2(SplitEditorOrthogonalAction);
registerAction2(SplitEditorLeftAction);
registerAction2(SplitEditorRightAction);
registerAction2(SplitEditorUpAction);
registerAction2(SplitEditorDownAction);
registerAction2(JoinTwoGroupsAction);
registerAction2(JoinAllGroupsAction);
registerAction2(NavigateBetweenGroupsAction);
registerAction2(ResetGroupSizesAction);
registerAction2(ToggleGroupSizesAction);
registerAction2(MaximizeGroupHideSidebarAction);
registerAction2(ToggleMaximizeEditorGroupAction);
registerAction2(MinimizeOtherGroupsAction);
registerAction2(MinimizeOtherGroupsHideSidebarAction);
registerAction2(MoveEditorLeftInGroupAction);
registerAction2(MoveEditorRightInGroupAction);
registerAction2(MoveGroupLeftAction);
registerAction2(MoveGroupRightAction);
registerAction2(MoveGroupUpAction);
registerAction2(MoveGroupDownAction);
registerAction2(DuplicateGroupLeftAction);
registerAction2(DuplicateGroupRightAction);
registerAction2(DuplicateGroupUpAction);
registerAction2(DuplicateGroupDownAction);
registerAction2(MoveEditorToPreviousGroupAction);
registerAction2(MoveEditorToNextGroupAction);
registerAction2(MoveEditorToFirstGroupAction);
registerAction2(MoveEditorToLastGroupAction);
registerAction2(MoveEditorToLeftGroupAction);
registerAction2(MoveEditorToRightGroupAction);
registerAction2(MoveEditorToAboveGroupAction);
registerAction2(MoveEditorToBelowGroupAction);
registerAction2(SplitEditorToPreviousGroupAction);
registerAction2(SplitEditorToNextGroupAction);
registerAction2(SplitEditorToFirstGroupAction);
registerAction2(SplitEditorToLastGroupAction);
registerAction2(SplitEditorToLeftGroupAction);
registerAction2(SplitEditorToRightGroupAction);
registerAction2(SplitEditorToAboveGroupAction);
registerAction2(SplitEditorToBelowGroupAction);
registerAction2(FocusActiveGroupAction);
registerAction2(FocusFirstGroupAction);
registerAction2(FocusLastGroupAction);
registerAction2(FocusPreviousGroup);
registerAction2(FocusNextGroup);
registerAction2(FocusLeftGroup);
registerAction2(FocusRightGroup);
registerAction2(FocusAboveGroup);
registerAction2(FocusBelowGroup);
registerAction2(NewEditorGroupLeftAction);
registerAction2(NewEditorGroupRightAction);
registerAction2(NewEditorGroupAboveAction);
registerAction2(NewEditorGroupBelowAction);
registerAction2(NavigatePreviousAction);
registerAction2(NavigateForwardInEditsAction);
registerAction2(NavigateBackwardsInEditsAction);
registerAction2(NavigatePreviousInEditsAction);
registerAction2(NavigateToLastEditLocationAction);
registerAction2(NavigateForwardInNavigationsAction);
registerAction2(NavigateBackwardsInNavigationsAction);
registerAction2(NavigatePreviousInNavigationsAction);
registerAction2(NavigateToLastNavigationLocationAction);
registerAction2(ClearEditorHistoryAction);
registerAction2(EditorLayoutSingleAction);
registerAction2(EditorLayoutTwoColumnsAction);
registerAction2(EditorLayoutThreeColumnsAction);
registerAction2(EditorLayoutTwoRowsAction);
registerAction2(EditorLayoutThreeRowsAction);
registerAction2(EditorLayoutTwoByTwoGridAction);
registerAction2(EditorLayoutTwoRowsRightAction);
registerAction2(EditorLayoutTwoColumnsBottomAction);
registerAction2(ToggleEditorTypeAction);
registerAction2(ReOpenInTextEditorAction);
registerAction2(QuickAccessPreviousRecentlyUsedEditorAction);
registerAction2(QuickAccessLeastRecentlyUsedEditorAction);
registerAction2(QuickAccessPreviousRecentlyUsedEditorInGroupAction);
registerAction2(QuickAccessLeastRecentlyUsedEditorInGroupAction);
registerAction2(QuickAccessPreviousEditorFromHistoryAction);
registerAction2(MoveEditorToNewWindowAction);
registerAction2(CopyEditorToNewindowAction);
registerAction2(MoveEditorGroupToNewWindowAction);
registerAction2(CopyEditorGroupToNewWindowAction);
registerAction2(RestoreEditorsToMainWindowAction);
registerAction2(NewEmptyEditorWindowAction);
const quickAccessNavigateNextInEditorPickerId = 'workbench.action.quickOpenNavigateNextInEditorPicker';
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: quickAccessNavigateNextInEditorPickerId,
    weight: 200  + 50,
    handler: getQuickNavigateHandler(quickAccessNavigateNextInEditorPickerId, true),
    when: editorPickerContext,
    primary: 2048  | 2 ,
    mac: { primary: 256  | 2  }
});
const quickAccessNavigatePreviousInEditorPickerId = 'workbench.action.quickOpenNavigatePreviousInEditorPicker';
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: quickAccessNavigatePreviousInEditorPickerId,
    weight: 200  + 50,
    handler: getQuickNavigateHandler(quickAccessNavigatePreviousInEditorPickerId, false),
    when: editorPickerContext,
    primary: 2048  | 1024  | 2 ,
    mac: { primary: 256  | 1024  | 2  }
});
setup();
if (isMacintosh) {
    MenuRegistry.appendMenuItem(MenuId.TouchBarContext, {
        command: { id: NavigateBackwardsAction.ID, title: NavigateBackwardsAction.LABEL, icon: { dark: ( (FileAccess.asFileUri('vs/workbench/browser/parts/editor/media/back-tb.png'))) } },
        group: 'navigation',
        order: 0
    });
    MenuRegistry.appendMenuItem(MenuId.TouchBarContext, {
        command: { id: NavigateForwardAction.ID, title: NavigateForwardAction.LABEL, icon: { dark: ( (FileAccess.asFileUri('vs/workbench/browser/parts/editor/media/forward-tb.png'))) } },
        group: 'navigation',
        order: 1
    });
}
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroup, { command: { id: LOCK_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 8, "Lock Group")), icon: Codicon.unlock }, group: 'navigation', order: 10, when: ( (ContextKeyExpr.and(
    IsAuxiliaryEditorPartContext,
     (ActiveEditorGroupLockedContext.toNegated())
))) });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroup, { command: { id: UNLOCK_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 9, "Unlock Group")), icon: Codicon.lock, toggled: ContextKeyExpr.true() }, group: 'navigation', order: 10, when: ActiveEditorGroupLockedContext });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroup, { command: { id: CLOSE_EDITOR_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 10, "Close Group")), icon: Codicon.close }, group: 'navigation', order: 20, when: ( (ContextKeyExpr.or(IsAuxiliaryEditorPartContext, EditorPartMultipleEditorGroupsContext))) });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroupContext, { command: { id: SPLIT_EDITOR_UP, title: ( localizeWithPath(_moduleId, 11, "Split Up")) }, group: '2_split', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroupContext, { command: { id: SPLIT_EDITOR_DOWN, title: ( localizeWithPath(_moduleId, 12, "Split Down")) }, group: '2_split', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroupContext, { command: { id: SPLIT_EDITOR_LEFT, title: ( localizeWithPath(_moduleId, 13, "Split Left")) }, group: '2_split', order: 30 });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroupContext, { command: { id: SPLIT_EDITOR_RIGHT, title: ( localizeWithPath(_moduleId, 14, "Split Right")) }, group: '2_split', order: 40 });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroupContext, { command: { id: NEW_EMPTY_EDITOR_WINDOW_COMMAND_ID, title: ( localizeWithPath(_moduleId, 15, "New Window")) }, group: '3_window', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroupContext, { command: { id: TOGGLE_LOCK_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 16, "Lock Group")), toggled: ActiveEditorGroupLockedContext }, group: '4_lock', order: 10, when: ( (IsAuxiliaryEditorPartContext.toNegated()))  });
MenuRegistry.appendMenuItem(MenuId.EmptyEditorGroupContext, { command: { id: CLOSE_EDITOR_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 17, "Close")) }, group: '5_close', order: 10, when: MultipleEditorGroupsContext });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { command: { id: SPLIT_EDITOR_UP, title: ( localizeWithPath(_moduleId, 11, "Split Up")) }, group: '2_split', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { command: { id: SPLIT_EDITOR_DOWN, title: ( localizeWithPath(_moduleId, 12, "Split Down")) }, group: '2_split', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { command: { id: SPLIT_EDITOR_LEFT, title: ( localizeWithPath(_moduleId, 13, "Split Left")) }, group: '2_split', order: 30 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { command: { id: SPLIT_EDITOR_RIGHT, title: ( localizeWithPath(_moduleId, 14, "Split Right")) }, group: '2_split', order: 40 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { command: { id: MOVE_EDITOR_GROUP_INTO_NEW_WINDOW_COMMAND_ID, title: ( localizeWithPath(_moduleId, 18, "Move into New Window")) }, group: '3_window', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { command: { id: COPY_EDITOR_GROUP_INTO_NEW_WINDOW_COMMAND_ID, title: ( localizeWithPath(_moduleId, 19, "Copy into New Window")) }, group: '3_window', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { submenu: MenuId.EditorTabsBarShowTabsSubmenu, title: ( localizeWithPath(_moduleId, 20, "Tab Bar")), group: '4_config', order: 10, when: ( (InEditorZenModeContext.negate())) });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarShowTabsSubmenu, { command: { id: ShowMultipleEditorTabsAction.ID, title: ( localizeWithPath(_moduleId, 21, "Multiple Tabs")), toggled: ( (ContextKeyExpr.equals('config.workbench.editor.showTabs', 'multiple'))) }, group: '1_config', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarShowTabsSubmenu, { command: { id: ShowSingleEditorTabAction.ID, title: ( localizeWithPath(_moduleId, 22, "Single Tab")), toggled: ( (ContextKeyExpr.equals('config.workbench.editor.showTabs', 'single'))) }, group: '1_config', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarShowTabsSubmenu, { command: { id: HideEditorTabsAction.ID, title: ( localizeWithPath(_moduleId, 23, "Hidden")), toggled: ( (ContextKeyExpr.equals('config.workbench.editor.showTabs', 'none'))) }, group: '1_config', order: 30 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { submenu: MenuId.EditorTabsBarShowTabsZenModeSubmenu, title: ( localizeWithPath(_moduleId, 20, "Tab Bar")), group: '4_config', order: 10, when: InEditorZenModeContext });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarShowTabsZenModeSubmenu, { command: { id: ZenShowMultipleEditorTabsAction.ID, title: ( localizeWithPath(_moduleId, 21, "Multiple Tabs")), toggled: ( (ContextKeyExpr.equals('config.zenMode.showTabs', 'multiple'))) }, group: '1_config', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarShowTabsZenModeSubmenu, { command: { id: ZenShowSingleEditorTabAction.ID, title: ( localizeWithPath(_moduleId, 22, "Single Tab")), toggled: ( (ContextKeyExpr.equals('config.zenMode.showTabs', 'single'))) }, group: '1_config', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarShowTabsZenModeSubmenu, { command: { id: ZenHideEditorTabsAction.ID, title: ( localizeWithPath(_moduleId, 23, "Hidden")), toggled: ( (ContextKeyExpr.equals('config.zenMode.showTabs', 'none'))) }, group: '1_config', order: 30 });
MenuRegistry.appendMenuItem(MenuId.EditorTabsBarContext, { submenu: MenuId.EditorActionsPositionSubmenu, title: ( localizeWithPath(_moduleId, 24, "Editor Actions Position")), group: '4_config', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorActionsPositionSubmenu, { command: { id: EditorActionsDefaultAction.ID, title: ( localizeWithPath(_moduleId, 20, "Tab Bar")), toggled: ( (ContextKeyExpr.equals('config.workbench.editor.editorActionsLocation', 'default'))) }, group: '1_config', order: 10, when: ( (( (ContextKeyExpr.equals('config.workbench.editor.showTabs', 'none'))).negate())) });
MenuRegistry.appendMenuItem(MenuId.EditorActionsPositionSubmenu, { command: { id: EditorActionsTitleBarAction.ID, title: ( localizeWithPath(_moduleId, 25, "Title Bar")), toggled: ( (ContextKeyExpr.or(
     (ContextKeyExpr.equals('config.workbench.editor.editorActionsLocation', 'titleBar')),
     (ContextKeyExpr.and(
         (ContextKeyExpr.equals('config.workbench.editor.showTabs', 'none')),
         (ContextKeyExpr.equals('config.workbench.editor.editorActionsLocation', 'default'))
    ))
))) }, group: '1_config', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorActionsPositionSubmenu, { command: { id: HideEditorActionsAction.ID, title: ( localizeWithPath(_moduleId, 26, "Hidden")), toggled: ( (ContextKeyExpr.equals('config.workbench.editor.editorActionsLocation', 'hidden'))) }, group: '1_config', order: 30 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: CLOSE_EDITOR_COMMAND_ID, title: ( localizeWithPath(_moduleId, 17, "Close")) }, group: '1_close', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: CLOSE_OTHER_EDITORS_IN_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 27, "Close Others")), precondition: ( (EditorGroupEditorsCountContext.notEqualsTo('1'))) }, group: '1_close', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: CLOSE_EDITORS_TO_THE_RIGHT_COMMAND_ID, title: ( localizeWithPath(_moduleId, 28, "Close to the Right")), precondition: ( (ActiveEditorLastInGroupContext.toNegated())) }, group: '1_close', order: 30, when: EditorTabsVisibleContext });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: CLOSE_SAVED_EDITORS_COMMAND_ID, title: ( localizeWithPath(_moduleId, 29, "Close Saved")) }, group: '1_close', order: 40 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: CLOSE_EDITORS_IN_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 30, "Close All")) }, group: '1_close', order: 50 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: REOPEN_WITH_COMMAND_ID, title: ( localizeWithPath(_moduleId, 31, "Reopen Editor With...")) }, group: '1_open', order: 10, when: ActiveEditorAvailableEditorIdsContext });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: KEEP_EDITOR_COMMAND_ID, title: ( localizeWithPath(_moduleId, 32, "Keep Open")), precondition: ( (ActiveEditorPinnedContext.toNegated())) }, group: '3_preview', order: 10, when: ( (ContextKeyExpr.has('config.workbench.editor.enablePreview'))) });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: PIN_EDITOR_COMMAND_ID, title: ( localizeWithPath(_moduleId, 33, "Pin")) }, group: '3_preview', order: 20, when: ( (ActiveEditorStickyContext.toNegated())) });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: UNPIN_EDITOR_COMMAND_ID, title: ( localizeWithPath(_moduleId, 34, "Unpin")) }, group: '3_preview', order: 20, when: ActiveEditorStickyContext });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: SPLIT_EDITOR_UP, title: ( localizeWithPath(_moduleId, 11, "Split Up")) }, group: '5_split', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: SPLIT_EDITOR_DOWN, title: ( localizeWithPath(_moduleId, 12, "Split Down")) }, group: '5_split', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: SPLIT_EDITOR_LEFT, title: ( localizeWithPath(_moduleId, 13, "Split Left")) }, group: '5_split', order: 30 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: SPLIT_EDITOR_RIGHT, title: ( localizeWithPath(_moduleId, 14, "Split Right")) }, group: '5_split', order: 40 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: SPLIT_EDITOR_IN_GROUP, title: ( localizeWithPath(_moduleId, 35, "Split in Group")) }, group: '6_split_in_group', order: 10, when: ActiveEditorCanSplitInGroupContext });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: JOIN_EDITOR_IN_GROUP, title: ( localizeWithPath(_moduleId, 36, "Join in Group")) }, group: '6_split_in_group', order: 10, when: SideBySideEditorActiveContext });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: MOVE_EDITOR_INTO_NEW_WINDOW_COMMAND_ID, title: ( localizeWithPath(_moduleId, 37, "Move into New Window")) }, group: '7_new_window', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTitleContext, { command: { id: COPY_EDITOR_INTO_NEW_WINDOW_COMMAND_ID, title: ( localizeWithPath(_moduleId, 38, "Copy into New Window")) }, group: '7_new_window', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { command: { id: TOGGLE_DIFF_SIDE_BY_SIDE, title: ( localizeWithPath(_moduleId, 39, "Inline View")), toggled: ( (ContextKeyExpr.equals('config.diffEditor.renderSideBySide', false))) }, group: '1_diff', order: 10, when: ( (ContextKeyExpr.has('isInDiffEditor'))) });
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { command: { id: SHOW_EDITORS_IN_GROUP, title: ( localizeWithPath(_moduleId, 40, "Show Opened Editors")) }, group: '3_open', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { command: { id: CLOSE_EDITORS_IN_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 30, "Close All")) }, group: '5_close', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { command: { id: CLOSE_SAVED_EDITORS_COMMAND_ID, title: ( localizeWithPath(_moduleId, 29, "Close Saved")) }, group: '5_close', order: 20 });
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { command: { id: TOGGLE_KEEP_EDITORS_COMMAND_ID, title: ( localizeWithPath(_moduleId, 41, "Enable Preview Editors")), toggled: ( (ContextKeyExpr.has('config.workbench.editor.enablePreview'))) }, group: '7_settings', order: 10 });
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { command: { id: TOGGLE_MAXIMIZE_EDITOR_GROUP, title: ( localizeWithPath(_moduleId, 42, "Maximize Group")) }, group: '8_group_operations', order: 5, when: ( (ContextKeyExpr.and(
     (EditorPartMaximizedEditorGroupContext.negate()),
    EditorPartMultipleEditorGroupsContext
))) });
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { command: { id: TOGGLE_MAXIMIZE_EDITOR_GROUP, title: ( localizeWithPath(_moduleId, 43, "Unmaximize Group")) }, group: '8_group_operations', order: 5, when: EditorPartMaximizedEditorGroupContext });
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { command: { id: TOGGLE_LOCK_GROUP_COMMAND_ID, title: ( localizeWithPath(_moduleId, 44, "Lock Group")), toggled: ActiveEditorGroupLockedContext }, group: '8_group_operations', order: 10, when: ( (IsAuxiliaryEditorPartContext.toNegated()))  });
function appendEditorToolItem(primary, when, order, alternative, precondition) {
    const item = {
        command: {
            id: primary.id,
            title: primary.title,
            icon: primary.icon,
            toggled: primary.toggled,
            precondition
        },
        group: 'navigation',
        when,
        order
    };
    if (alternative) {
        item.alt = {
            id: alternative.id,
            title: alternative.title,
            icon: alternative.icon
        };
    }
    MenuRegistry.appendMenuItem(MenuId.EditorTitle, item);
}
const SPLIT_ORDER = 100000;
const CLOSE_ORDER = 1000000;
appendEditorToolItem({
    id: SPLIT_EDITOR,
    title: ( localizeWithPath(_moduleId, 45, "Split Editor Right")),
    icon: Codicon.splitHorizontal
}, ContextKeyExpr.not('splitEditorsVertically'), SPLIT_ORDER, {
    id: SPLIT_EDITOR_DOWN,
    title: ( localizeWithPath(_moduleId, 46, "Split Editor Down")),
    icon: Codicon.splitVertical
});
appendEditorToolItem({
    id: SPLIT_EDITOR,
    title: ( localizeWithPath(_moduleId, 46, "Split Editor Down")),
    icon: Codicon.splitVertical
}, ( (ContextKeyExpr.has('splitEditorsVertically'))), SPLIT_ORDER, {
    id: SPLIT_EDITOR_RIGHT,
    title: ( localizeWithPath(_moduleId, 45, "Split Editor Right")),
    icon: Codicon.splitHorizontal
});
appendEditorToolItem({
    id: TOGGLE_SPLIT_EDITOR_IN_GROUP_LAYOUT,
    title: ( localizeWithPath(_moduleId, 47, "Toggle Layout")),
    icon: Codicon.editorLayout
}, SideBySideEditorActiveContext, SPLIT_ORDER - 1);
appendEditorToolItem({
    id: CLOSE_EDITOR_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 17, "Close")),
    icon: Codicon.close
}, ( (ContextKeyExpr.and(
     (EditorTabsVisibleContext.toNegated()),
     (ActiveEditorDirtyContext.toNegated()),
     (ActiveEditorStickyContext.toNegated())
))), CLOSE_ORDER, {
    id: CLOSE_EDITORS_IN_GROUP_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 30, "Close All")),
    icon: Codicon.closeAll
});
appendEditorToolItem({
    id: CLOSE_EDITOR_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 17, "Close")),
    icon: Codicon.closeDirty
}, ( (ContextKeyExpr.and(
     (EditorTabsVisibleContext.toNegated()),
    ActiveEditorDirtyContext,
     (ActiveEditorStickyContext.toNegated())
))), CLOSE_ORDER, {
    id: CLOSE_EDITORS_IN_GROUP_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 30, "Close All")),
    icon: Codicon.closeAll
});
appendEditorToolItem({
    id: UNPIN_EDITOR_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 34, "Unpin")),
    icon: Codicon.pinned
}, ( (ContextKeyExpr.and(
     (EditorTabsVisibleContext.toNegated()),
     (ActiveEditorDirtyContext.toNegated()),
    ActiveEditorStickyContext
))), CLOSE_ORDER, {
    id: CLOSE_EDITOR_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 17, "Close")),
    icon: Codicon.close
});
appendEditorToolItem({
    id: UNPIN_EDITOR_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 34, "Unpin")),
    icon: Codicon.pinnedDirty
}, ( (ContextKeyExpr.and(
     (EditorTabsVisibleContext.toNegated()),
    ActiveEditorDirtyContext,
    ActiveEditorStickyContext
))), CLOSE_ORDER, {
    id: CLOSE_EDITOR_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 17, "Close")),
    icon: Codicon.close
});
appendEditorToolItem({
    id: LOCK_GROUP_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 48, "Lock Group")),
    icon: Codicon.unlock
}, ( (ContextKeyExpr.and(
    IsAuxiliaryEditorPartContext,
     (ActiveEditorGroupLockedContext.toNegated())
))), CLOSE_ORDER - 1);
appendEditorToolItem({
    id: UNLOCK_GROUP_COMMAND_ID,
    title: ( localizeWithPath(_moduleId, 49, "Unlock Group")),
    icon: Codicon.lock,
    toggled: ContextKeyExpr.true()
}, ActiveEditorGroupLockedContext, CLOSE_ORDER - 1);
const previousChangeIcon = registerIcon('diff-editor-previous-change', Codicon.arrowUp, ( localizeWithPath(_moduleId, 50, 'Icon for the previous change action in the diff editor.')));
appendEditorToolItem({
    id: GOTO_PREVIOUS_CHANGE,
    title: ( localizeWithPath(_moduleId, 51, "Previous Change")),
    icon: previousChangeIcon
}, TextCompareEditorActiveContext, 10, undefined, EditorContextKeys.hasChanges);
const nextChangeIcon = registerIcon('diff-editor-next-change', Codicon.arrowDown, ( localizeWithPath(_moduleId, 52, 'Icon for the next change action in the diff editor.')));
appendEditorToolItem({
    id: GOTO_NEXT_CHANGE,
    title: ( localizeWithPath(_moduleId, 53, "Next Change")),
    icon: nextChangeIcon
}, TextCompareEditorActiveContext, 11, undefined, EditorContextKeys.hasChanges);
appendEditorToolItem({
    id: DIFF_SWAP_SIDES,
    title: ( localizeWithPath(_moduleId, 54, "Swap Left and Right Side")),
    icon: Codicon.arrowSwap
}, ( (ContextKeyExpr.and(TextCompareEditorActiveContext, ActiveCompareEditorCanSwapContext))), 15, undefined, undefined);
const toggleWhitespace = registerIcon('diff-editor-toggle-whitespace', Codicon.whitespace, ( localizeWithPath(_moduleId, 55, 'Icon for the toggle whitespace action in the diff editor.')));
MenuRegistry.appendMenuItem(MenuId.EditorTitle, {
    command: {
        id: TOGGLE_DIFF_IGNORE_TRIM_WHITESPACE,
        title: ( localizeWithPath(_moduleId, 56, "Show Leading/Trailing Whitespace Differences")),
        icon: toggleWhitespace,
        precondition: TextCompareEditorActiveContext,
        toggled: ( (ContextKeyExpr.equals('config.diffEditor.ignoreTrimWhitespace', false))),
    },
    group: 'navigation',
    when: TextCompareEditorActiveContext,
    order: 20,
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: KEEP_EDITOR_COMMAND_ID, title: ( localize2WithPath(_moduleId, 57, 'Keep Editor')), category: Categories.View }, when: ( (ContextKeyExpr.has('config.workbench.editor.enablePreview'))) });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: PIN_EDITOR_COMMAND_ID, title: ( localize2WithPath(_moduleId, 58, 'Pin Editor')), category: Categories.View } });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: UNPIN_EDITOR_COMMAND_ID, title: ( localize2WithPath(_moduleId, 59, 'Unpin Editor')), category: Categories.View } });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: CLOSE_EDITOR_COMMAND_ID, title: ( localize2WithPath(_moduleId, 60, 'Close Editor')), category: Categories.View } });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: CLOSE_PINNED_EDITOR_COMMAND_ID, title: ( localize2WithPath(_moduleId, 61, 'Close Pinned Editor')), category: Categories.View } });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: CLOSE_EDITORS_IN_GROUP_COMMAND_ID, title: ( localize2WithPath(_moduleId, 62, 'Close All Editors in Group')), category: Categories.View } });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: CLOSE_SAVED_EDITORS_COMMAND_ID, title: ( localize2WithPath(_moduleId, 63, 'Close Saved Editors in Group')), category: Categories.View } });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: CLOSE_OTHER_EDITORS_IN_GROUP_COMMAND_ID, title: ( localize2WithPath(_moduleId, 64, 'Close Other Editors in Group')), category: Categories.View } });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: CLOSE_EDITORS_TO_THE_RIGHT_COMMAND_ID, title: ( localize2WithPath(_moduleId, 65, 'Close Editors to the Right in Group')), category: Categories.View }, when: ( (ActiveEditorLastInGroupContext.toNegated())) });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: CLOSE_EDITORS_AND_GROUP_COMMAND_ID, title: ( localize2WithPath(_moduleId, 66, 'Close Editor Group')), category: Categories.View }, when: MultipleEditorGroupsContext });
MenuRegistry.appendMenuItem(MenuId.CommandPalette, { command: { id: REOPEN_WITH_COMMAND_ID, title: ( localize2WithPath(_moduleId, 31, "Reopen Editor With...")), category: Categories.View }, when: ActiveEditorAvailableEditorIdsContext });
MenuRegistry.appendMenuItem(MenuId.MenubarRecentMenu, {
    group: '1_editor',
    command: {
        id: ReopenClosedEditorAction.ID,
        title: ( localizeWithPath(_moduleId, 67, "&&Reopen Closed Editor")),
        precondition: ( (ContextKeyExpr.has('canReopenClosedEditor')))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarRecentMenu, {
    group: 'z_clear',
    command: {
        id: ClearRecentFilesAction.ID,
        title: ( localizeWithPath(_moduleId, 68, "&&Clear Recently Opened..."))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarFileMenu, {
    title: ( localizeWithPath(_moduleId, 69, "Share")),
    submenu: MenuId.MenubarShare,
    group: '45_share',
    order: 1,
});
MenuRegistry.appendMenuItem(MenuId.MenubarViewMenu, {
    group: '2_appearance',
    title: ( localizeWithPath(_moduleId, 70, "Editor &&Layout")),
    submenu: MenuId.MenubarLayoutMenu,
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '1_split',
    command: {
        id: SPLIT_EDITOR_UP,
        title: {
            ...( localize2WithPath(_moduleId, 71, "Split Up")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 72, "Split &&Up")),
        }
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '1_split',
    command: {
        id: SPLIT_EDITOR_DOWN,
        title: {
            ...( localize2WithPath(_moduleId, 73, "Split Down")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 74, "Split &&Down")),
        }
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '1_split',
    command: {
        id: SPLIT_EDITOR_LEFT,
        title: {
            ...( localize2WithPath(_moduleId, 75, "Split Left")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 76, "Split &&Left")),
        }
    },
    order: 3
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '1_split',
    command: {
        id: SPLIT_EDITOR_RIGHT,
        title: {
            ...( localize2WithPath(_moduleId, 77, "Split Right")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 78, "Split &&Right")),
        }
    },
    order: 4
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '2_split_in_group',
    command: {
        id: SPLIT_EDITOR_IN_GROUP,
        title: {
            ...( localize2WithPath(_moduleId, 79, "Split in Group")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 80, "Split in &&Group")),
        }
    },
    when: ActiveEditorCanSplitInGroupContext,
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '2_split_in_group',
    command: {
        id: JOIN_EDITOR_IN_GROUP,
        title: {
            ...( localize2WithPath(_moduleId, 81, "Join in Group")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 82, "Join in &&Group")),
        }
    },
    when: SideBySideEditorActiveContext,
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '3_new_window',
    command: {
        id: MOVE_EDITOR_INTO_NEW_WINDOW_COMMAND_ID,
        title: {
            ...( localize2WithPath(_moduleId, 83, "Move Editor into New Window")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 84, "&&Move Editor into New Window")),
        }
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '3_new_window',
    command: {
        id: COPY_EDITOR_INTO_NEW_WINDOW_COMMAND_ID,
        title: {
            ...( localize2WithPath(_moduleId, 85, "Copy Editor into New Window")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 86, "&&Copy Editor into New Window")),
        }
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '4_layouts',
    command: {
        id: EditorLayoutSingleAction.ID,
        title: {
            ...( localize2WithPath(_moduleId, 87, "Single")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 88, "&&Single")),
        }
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '4_layouts',
    command: {
        id: EditorLayoutTwoColumnsAction.ID,
        title: {
            ...( localize2WithPath(_moduleId, 89, "Two Columns")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 90, "&&Two Columns")),
        }
    },
    order: 3
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '4_layouts',
    command: {
        id: EditorLayoutThreeColumnsAction.ID,
        title: {
            ...( localize2WithPath(_moduleId, 91, "Three Columns")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 92, "T&&hree Columns")),
        }
    },
    order: 4
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '4_layouts',
    command: {
        id: EditorLayoutTwoRowsAction.ID,
        title: {
            ...( localize2WithPath(_moduleId, 93, "Two Rows")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 94, "T&&wo Rows")),
        }
    },
    order: 5
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '4_layouts',
    command: {
        id: EditorLayoutThreeRowsAction.ID,
        title: {
            ...( localize2WithPath(_moduleId, 95, "Three Rows")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 96, "Three &&Rows")),
        }
    },
    order: 6
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '4_layouts',
    command: {
        id: EditorLayoutTwoByTwoGridAction.ID,
        title: {
            ...( localize2WithPath(_moduleId, 97, "Grid (2x2)")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 98, "&&Grid (2x2)")),
        }
    },
    order: 7
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '4_layouts',
    command: {
        id: EditorLayoutTwoRowsRightAction.ID,
        title: {
            ...( localize2WithPath(_moduleId, 99, "Two Rows Right")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 100, "Two R&&ows Right")),
        }
    },
    order: 8
});
MenuRegistry.appendMenuItem(MenuId.MenubarLayoutMenu, {
    group: '4_layouts',
    command: {
        id: EditorLayoutTwoColumnsBottomAction.ID,
        title: {
            ...( localize2WithPath(_moduleId, 101, "Two Columns Bottom")),
            mnemonicTitle: ( localizeWithPath(_moduleId, 102, "Two &&Columns Bottom")),
        }
    },
    order: 9
});
MenuRegistry.appendMenuItem(MenuId.MenubarGoMenu, {
    group: '1_history_nav',
    command: {
        id: 'workbench.action.navigateToLastEditLocation',
        title: ( localizeWithPath(_moduleId, 103, "&&Last Edit Location")),
        precondition: ( (ContextKeyExpr.has('canNavigateToLastEditLocation')))
    },
    order: 3
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '1_sideBySide',
    command: {
        id: FOCUS_FIRST_SIDE_EDITOR,
        title: ( localizeWithPath(_moduleId, 104, "&&First Side in Editor"))
    },
    when: ( (ContextKeyExpr.or(SideBySideEditorActiveContext, TextCompareEditorActiveContext))),
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '1_sideBySide',
    command: {
        id: FOCUS_SECOND_SIDE_EDITOR,
        title: ( localizeWithPath(_moduleId, 105, "&&Second Side in Editor"))
    },
    when: ( (ContextKeyExpr.or(SideBySideEditorActiveContext, TextCompareEditorActiveContext))),
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '2_any',
    command: {
        id: 'workbench.action.nextEditor',
        title: ( localizeWithPath(_moduleId, 106, "&&Next Editor"))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '2_any',
    command: {
        id: 'workbench.action.previousEditor',
        title: ( localizeWithPath(_moduleId, 107, "&&Previous Editor"))
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '3_any_used',
    command: {
        id: 'workbench.action.openNextRecentlyUsedEditor',
        title: ( localizeWithPath(_moduleId, 108, "&&Next Used Editor"))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '3_any_used',
    command: {
        id: 'workbench.action.openPreviousRecentlyUsedEditor',
        title: ( localizeWithPath(_moduleId, 109, "&&Previous Used Editor"))
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '4_group',
    command: {
        id: 'workbench.action.nextEditorInGroup',
        title: ( localizeWithPath(_moduleId, 110, "&&Next Editor in Group"))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '4_group',
    command: {
        id: 'workbench.action.previousEditorInGroup',
        title: ( localizeWithPath(_moduleId, 111, "&&Previous Editor in Group"))
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '5_group_used',
    command: {
        id: 'workbench.action.openNextRecentlyUsedEditorInGroup',
        title: ( localizeWithPath(_moduleId, 112, "&&Next Used Editor in Group"))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchEditorMenu, {
    group: '5_group_used',
    command: {
        id: 'workbench.action.openPreviousRecentlyUsedEditorInGroup',
        title: ( localizeWithPath(_moduleId, 113, "&&Previous Used Editor in Group"))
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarGoMenu, {
    group: '2_editor_nav',
    title: ( localizeWithPath(_moduleId, 114, "Switch &&Editor")),
    submenu: MenuId.MenubarSwitchEditorMenu,
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '1_focus_index',
    command: {
        id: 'workbench.action.focusFirstEditorGroup',
        title: ( localizeWithPath(_moduleId, 115, "Group &&1"))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '1_focus_index',
    command: {
        id: 'workbench.action.focusSecondEditorGroup',
        title: ( localizeWithPath(_moduleId, 116, "Group &&2"))
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '1_focus_index',
    command: {
        id: 'workbench.action.focusThirdEditorGroup',
        title: ( localizeWithPath(_moduleId, 117, "Group &&3")),
        precondition: MultipleEditorGroupsContext
    },
    order: 3
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '1_focus_index',
    command: {
        id: 'workbench.action.focusFourthEditorGroup',
        title: ( localizeWithPath(_moduleId, 118, "Group &&4")),
        precondition: MultipleEditorGroupsContext
    },
    order: 4
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '1_focus_index',
    command: {
        id: 'workbench.action.focusFifthEditorGroup',
        title: ( localizeWithPath(_moduleId, 119, "Group &&5")),
        precondition: MultipleEditorGroupsContext
    },
    order: 5
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '2_next_prev',
    command: {
        id: 'workbench.action.focusNextGroup',
        title: ( localizeWithPath(_moduleId, 120, "&&Next Group")),
        precondition: MultipleEditorGroupsContext
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '2_next_prev',
    command: {
        id: 'workbench.action.focusPreviousGroup',
        title: ( localizeWithPath(_moduleId, 121, "&&Previous Group")),
        precondition: MultipleEditorGroupsContext
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '3_directional',
    command: {
        id: 'workbench.action.focusLeftGroup',
        title: ( localizeWithPath(_moduleId, 122, "Group &&Left")),
        precondition: MultipleEditorGroupsContext
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '3_directional',
    command: {
        id: 'workbench.action.focusRightGroup',
        title: ( localizeWithPath(_moduleId, 123, "Group &&Right")),
        precondition: MultipleEditorGroupsContext
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '3_directional',
    command: {
        id: 'workbench.action.focusAboveGroup',
        title: ( localizeWithPath(_moduleId, 124, "Group &&Above")),
        precondition: MultipleEditorGroupsContext
    },
    order: 3
});
MenuRegistry.appendMenuItem(MenuId.MenubarSwitchGroupMenu, {
    group: '3_directional',
    command: {
        id: 'workbench.action.focusBelowGroup',
        title: ( localizeWithPath(_moduleId, 125, "Group &&Below")),
        precondition: MultipleEditorGroupsContext
    },
    order: 4
});
MenuRegistry.appendMenuItem(MenuId.MenubarGoMenu, {
    group: '2_editor_nav',
    title: ( localizeWithPath(_moduleId, 126, "Switch &&Group")),
    submenu: MenuId.MenubarSwitchGroupMenu,
    order: 2
});
