import './media/style.css.js';
import { registerThemingParticipant } from 'vscode/vscode/vs/platform/theme/common/themeService';
import { WORKBENCH_BACKGROUND, TITLE_BAR_ACTIVE_BACKGROUND } from 'vscode/vscode/vs/workbench/common/theme';
import { isWeb, isIOS } from 'vscode/vscode/vs/base/common/platform';
import { createMetaElement } from 'vscode/vscode/vs/base/browser/dom';
import { isSafari, isStandalone } from 'vscode/vscode/vs/base/browser/browser';
import 'vscode/vscode/vs/platform/theme/common/colorUtils';
import { selectionBackground } from 'vscode/vscode/vs/platform/theme/common/colors/baseColors';
import 'vscode/vscode/vs/platform/theme/common/colors/chartsColors';
import 'vscode/vscode/vs/platform/theme/common/colors/editorColors';
import 'vscode/vscode/vs/platform/theme/common/colors/inputColors';
import 'vscode/vscode/vs/platform/theme/common/colors/listColors';
import 'vscode/vscode/vs/platform/theme/common/colors/menuColors';
import 'vscode/vscode/vs/platform/theme/common/colors/minimapColors';
import 'vscode/vscode/vs/platform/theme/common/colors/miscColors';
import 'vscode/vscode/vs/platform/theme/common/colors/quickpickColors';
import 'vscode/vscode/vs/platform/theme/common/colors/searchColors';
import { mainWindow } from 'vscode/vscode/vs/base/browser/window';

registerThemingParticipant((theme, collector) => {
    const workbenchBackground = WORKBENCH_BACKGROUND(theme);
    collector.addRule(`.monaco-workbench-part { background-color: ${workbenchBackground}; }`);
    const windowSelectionBackground = theme.getColor(selectionBackground);
    if (windowSelectionBackground) {
        collector.addRule(`.monaco-workbench-part ::selection { background-color: ${windowSelectionBackground}; }`);
    }
    if (isWeb) {
        const titleBackground = theme.getColor(TITLE_BAR_ACTIVE_BACKGROUND);
        if (titleBackground) {
            const metaElementId = 'monaco-workbench-meta-theme-color';
            let metaElement = mainWindow.document.getElementById(metaElementId);
            if (!metaElement) {
                metaElement = createMetaElement();
                metaElement.name = 'theme-color';
                metaElement.id = metaElementId;
            }
            metaElement.content = ( titleBackground.toString());
        }
    }
    if (isSafari) {
        collector.addRule(`
			.monaco-workbench-part.web {
				touch-action: none;
			}
			.monaco-workbench-part .monaco-editor .view-lines {
				user-select: text;
				-webkit-user-select: text;
			}
		`);
    }
    if (isIOS && isStandalone()) {
        collector.addRule(`.monaco-workbench-part { background-color: ${workbenchBackground}; }`);
    }
});
