import { EditorContextKeys } from 'vscode/vscode/vs/editor/common/editorContextKeys';
import { localizeWithPath, localize2WithPath } from 'vscode/vscode/vs/nls';
import { Action2, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { Categories } from 'vscode/vscode/vs/platform/action/common/actionCommonCategories';
import { KEYBINDING_CONTEXT_WEBVIEW_FIND_WIDGET_ENABLED, KEYBINDING_CONTEXT_WEBVIEW_FIND_WIDGET_VISIBLE, KEYBINDING_CONTEXT_WEBVIEW_FIND_WIDGET_FOCUSED } from 'vscode/vscode/vs/workbench/contrib/webview/browser/webview';
import { IWebviewService } from 'vscode/vscode/vs/workbench/contrib/webview/browser/webview.service';
import { WebviewEditor } from 'vscode/vscode/vs/workbench/contrib/webviewPanel/browser/webviewEditor';
import { WebviewInput } from 'vscode/vscode/vs/workbench/contrib/webviewPanel/browser/webviewEditorInput';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService.service';

const _moduleId = "vs/workbench/contrib/webviewPanel/browser/webviewCommands";
const webviewActiveContextKeyExpr = ( (ContextKeyExpr.and(
     (ContextKeyExpr.equals('activeEditor', WebviewEditor.ID)),
     (EditorContextKeys.focus.toNegated())
)));
class ShowWebViewEditorFindWidgetAction extends Action2 {
    static { this.ID = 'editor.action.webvieweditor.showFind'; }
    static { this.LABEL = ( localizeWithPath(_moduleId, 0, "Show find")); }
    constructor() {
        super({
            id: ShowWebViewEditorFindWidgetAction.ID,
            title: ShowWebViewEditorFindWidgetAction.LABEL,
            keybinding: {
                when: ( (ContextKeyExpr.and(
                    webviewActiveContextKeyExpr,
                    KEYBINDING_CONTEXT_WEBVIEW_FIND_WIDGET_ENABLED
                ))),
                primary: 2048  | 36 ,
                weight: 100
            }
        });
    }
    run(accessor) {
        getActiveWebviewEditor(accessor)?.showFind();
    }
}
class HideWebViewEditorFindCommand extends Action2 {
    static { this.ID = 'editor.action.webvieweditor.hideFind'; }
    static { this.LABEL = ( localizeWithPath(_moduleId, 1, "Stop find")); }
    constructor() {
        super({
            id: HideWebViewEditorFindCommand.ID,
            title: HideWebViewEditorFindCommand.LABEL,
            keybinding: {
                when: ( (ContextKeyExpr.and(
                    webviewActiveContextKeyExpr,
                    KEYBINDING_CONTEXT_WEBVIEW_FIND_WIDGET_VISIBLE
                ))),
                primary: 9 ,
                weight: 100
            }
        });
    }
    run(accessor) {
        getActiveWebviewEditor(accessor)?.hideFind();
    }
}
class WebViewEditorFindNextCommand extends Action2 {
    static { this.ID = 'editor.action.webvieweditor.findNext'; }
    static { this.LABEL = ( localizeWithPath(_moduleId, 2, 'Find next')); }
    constructor() {
        super({
            id: WebViewEditorFindNextCommand.ID,
            title: WebViewEditorFindNextCommand.LABEL,
            keybinding: {
                when: ( (ContextKeyExpr.and(
                    webviewActiveContextKeyExpr,
                    KEYBINDING_CONTEXT_WEBVIEW_FIND_WIDGET_FOCUSED
                ))),
                primary: 3 ,
                weight: 100
            }
        });
    }
    run(accessor) {
        getActiveWebviewEditor(accessor)?.runFindAction(false);
    }
}
class WebViewEditorFindPreviousCommand extends Action2 {
    static { this.ID = 'editor.action.webvieweditor.findPrevious'; }
    static { this.LABEL = ( localizeWithPath(_moduleId, 3, 'Find previous')); }
    constructor() {
        super({
            id: WebViewEditorFindPreviousCommand.ID,
            title: WebViewEditorFindPreviousCommand.LABEL,
            keybinding: {
                when: ( (ContextKeyExpr.and(
                    webviewActiveContextKeyExpr,
                    KEYBINDING_CONTEXT_WEBVIEW_FIND_WIDGET_FOCUSED
                ))),
                primary: 1024  | 3 ,
                weight: 100
            }
        });
    }
    run(accessor) {
        getActiveWebviewEditor(accessor)?.runFindAction(true);
    }
}
class ReloadWebviewAction extends Action2 {
    static { this.ID = 'workbench.action.webview.reloadWebviewAction'; }
    static { this.LABEL = ( localize2WithPath(_moduleId, 4, "Reload Webviews")); }
    constructor() {
        super({
            id: ReloadWebviewAction.ID,
            title: ReloadWebviewAction.LABEL,
            category: Categories.Developer,
            menu: [{
                    id: MenuId.CommandPalette
                }]
        });
    }
    async run(accessor) {
        const webviewService = accessor.get(IWebviewService);
        for (const webview of webviewService.webviews) {
            webview.reload();
        }
    }
}
function getActiveWebviewEditor(accessor) {
    const editorService = accessor.get(IEditorService);
    const activeEditor = editorService.activeEditor;
    return activeEditor instanceof WebviewInput ? activeEditor.webview : undefined;
}

export { HideWebViewEditorFindCommand, ReloadWebviewAction, ShowWebViewEditorFindWidgetAction, WebViewEditorFindNextCommand, WebViewEditorFindPreviousCommand };
