import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { IActivityService } from 'vscode/vscode/vs/workbench/services/activity/common/activity.service';
import { Disposable, toDisposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { IViewDescriptorService } from 'vscode/vscode/vs/workbench/common/views.service';
import { ACCOUNTS_ACTIVITY_ID, GLOBAL_ACTIVITY_ID } from 'vscode/vscode/vs/workbench/common/activity';
import { Event, Emitter } from 'vscode/vscode/vs/base/common/event';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { isUndefined } from 'vscode/vscode/vs/base/common/types';

let ViewContainerActivityByView = class ViewContainerActivityByView extends Disposable {
    constructor(viewId, viewDescriptorService, activityService) {
        super();
        this.viewId = viewId;
        this.viewDescriptorService = viewDescriptorService;
        this.activityService = activityService;
        this.activity = undefined;
        this.activityDisposable = Disposable.None;
        this._register(Event.filter(this.viewDescriptorService.onDidChangeContainer, e => ( e.views.some(view => view.id === viewId)))(() => this.update()));
        this._register(Event.filter(this.viewDescriptorService.onDidChangeLocation, e => ( e.views.some(view => view.id === viewId)))(() => this.update()));
    }
    setActivity(activity) {
        this.activity = activity;
        this.update();
    }
    clearActivity() {
        this.activity = undefined;
        this.update();
    }
    update() {
        this.activityDisposable.dispose();
        const container = this.viewDescriptorService.getViewContainerByViewId(this.viewId);
        if (container && this.activity) {
            this.activityDisposable = this.activityService.showViewContainerActivity(container.id, this.activity);
        }
    }
    dispose() {
        this.activityDisposable.dispose();
        super.dispose();
    }
};
ViewContainerActivityByView = ( __decorate([
    ( __param(1, IViewDescriptorService)),
    ( __param(2, IActivityService))
], ViewContainerActivityByView));
let ActivityService = class ActivityService extends Disposable {
    constructor(viewDescriptorService, instantiationService) {
        super();
        this.viewDescriptorService = viewDescriptorService;
        this.instantiationService = instantiationService;
        this.viewActivities = ( new Map());
        this._onDidChangeActivity = this._register(( new Emitter()));
        this.onDidChangeActivity = this._onDidChangeActivity.event;
        this.viewContainerActivities = ( new Map());
        this.globalActivities = ( new Map());
    }
    showViewContainerActivity(viewContainerId, activity) {
        const viewContainer = this.viewDescriptorService.getViewContainerById(viewContainerId);
        if (viewContainer) {
            let activities = this.viewContainerActivities.get(viewContainerId);
            if (!activities) {
                activities = [];
                this.viewContainerActivities.set(viewContainerId, activities);
            }
            for (let i = 0; i <= activities.length; i++) {
                if (i === activities.length || isUndefined(activity.priority)) {
                    activities.push(activity);
                    break;
                }
                else if (isUndefined(activities[i].priority) || activities[i].priority <= activity.priority) {
                    activities.splice(i, 0, activity);
                    break;
                }
            }
            this._onDidChangeActivity.fire(viewContainer);
            return toDisposable(() => {
                activities.splice(activities.indexOf(activity), 1);
                if (activities.length === 0) {
                    this.viewContainerActivities.delete(viewContainerId);
                }
                this._onDidChangeActivity.fire(viewContainer);
            });
        }
        return Disposable.None;
    }
    getViewContainerActivities(viewContainerId) {
        const viewContainer = this.viewDescriptorService.getViewContainerById(viewContainerId);
        if (viewContainer) {
            return this.viewContainerActivities.get(viewContainerId) ?? [];
        }
        return [];
    }
    showViewActivity(viewId, activity) {
        let maybeItem = this.viewActivities.get(viewId);
        if (maybeItem) {
            maybeItem.id++;
        }
        else {
            maybeItem = {
                id: 1,
                activity: this.instantiationService.createInstance(ViewContainerActivityByView, viewId)
            };
            this.viewActivities.set(viewId, maybeItem);
        }
        const id = maybeItem.id;
        maybeItem.activity.setActivity(activity);
        const item = maybeItem;
        return toDisposable(() => {
            if (item.id === id) {
                item.activity.dispose();
                this.viewActivities.delete(viewId);
            }
        });
    }
    showAccountsActivity(activity) {
        return this.showActivity(ACCOUNTS_ACTIVITY_ID, activity);
    }
    showGlobalActivity(activity) {
        return this.showActivity(GLOBAL_ACTIVITY_ID, activity);
    }
    getActivity(id) {
        return this.globalActivities.get(id) ?? [];
    }
    showActivity(id, activity) {
        let activities = this.globalActivities.get(id);
        if (!activities) {
            activities = [];
            this.globalActivities.set(id, activities);
        }
        activities.push(activity);
        this._onDidChangeActivity.fire(id);
        return toDisposable(() => {
            activities.splice(activities.indexOf(activity), 1);
            if (activities.length === 0) {
                this.globalActivities.delete(id);
            }
            this._onDidChangeActivity.fire(id);
        });
    }
};
ActivityService = ( __decorate([
    ( __param(0, IViewDescriptorService)),
    ( __param(1, IInstantiationService))
], ActivityService));

export { ActivityService };
